import datetime
import functools
import json
import random
import string

from tornado import testing
from tornado.ioloop import IOLoop
from tornado.platform.asyncio import AsyncIOMainLoop


class BaseAPITest(testing.AsyncHTTPTestCase):

    def get_new_ioloop(self):
        self._io_loop = AsyncIOMainLoop()
        return self._io_loop

    def get_app(self):
        app = self.get_flasky_app()
        app.build_app()

        for on_start_func in app.on_start_funcs:
            IOLoop.current().run_sync(functools.partial(on_start_func, app))

        return app.app

    def get_flasky_app(self):
        raise NotImplemented

    def load_body(self, response):
        return json.loads(response.body.decode("utf-8"))

    def generate_random_string(self, N):
        return ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(N))

    def parse_datetime(self, value):
        return datetime.datetime.strptime(value, "%Y-%m-%dT%H:%M:%S.%fZ")


import logging

from quark_utilities import temporal_helpers

from quark_utilities import mongol

logger = logging.getLogger("quark_utilities.event_hub")
hub = None

def init_event_hub(app, event_store=None):
    global hub
    hub = EventHub(app)

class Event(object):
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

class DefaultEventStore(mongol.Repository):
    pass

class EventHub(object):

    def __init__(self, app):
        self._app = app
        self._handlers = {}
        self._event_store = None
        self._app.on_start(self._init_event_store)

    async def _init_event_store(self, app):
        if app.settings.get("save_events", False):
            logger.info("Creating event store.")
            db = await app.di.get("db")
            self._event_store = DefaultEventStore(db["events"])

    def subscribe(self, event_type):
        if event_type and event_type not in self._handlers:
            self._handlers[event_type] = set()

        def wrapper(f):
            self._handlers[event_type].add(f)
            return f

        return wrapper

    async def dispatch(self, event):
        logger.info("Event arrived <{}>".format(str(type(event))))

        handlers = self._handlers.get(type(event), [])
        for handler in handlers:
            await handler(event)

        if self._event_store:
            data = {
                "type": event.__class__.__name__,
            }

            data.update(event.__dict__)
            data["created_at"] = temporal_helpers.utc_now()
            await self._event_store.save(data)







import os
import sys
import jwt
import datetime
from calendar import timegm

import pytest
from unittest import mock

from flasky import InvalidTokenError

sys.path.insert(0, os.path.dirname(os.path.abspath(__file__ + "/../")))

from quark_utilities import jwt_security

mock_app = mock.MagicMock()
mock_handler = mock.MagicMock()
secret = "secret"
manager = jwt_security.SecurityManager(mock_app, secret, issuer="Quark-CMS")

@pytest.mark.asyncio
async def test_should_raise_none_when_method_definition_is_not_contain_secure_field():
    ret = await manager._before_request_hook(mock_handler, {})
    assert ret is None

@pytest.mark.asyncio
async def test_should_raise_not_authorized_exc_when_auth_header_not_found():
    mock_handler.request.headers = {}
    try:
        ret = await manager._before_request_hook(mock_handler, {"secure":["admin"]})
    except jwt_security.AgentIsNotAuthorized as e:
        assert e.message.startswith("Authorization header is")
        return

    assert False


@pytest.mark.asyncio
async def test_should_raise_not_authorized_exc_when_auth_header_is_not_bearer_typ():
    mock_handler.request.headers = {"Authorization": "Basic vıdıvıdı"}

    try:
        ret = await manager._before_request_hook(mock_handler, {"secure":["admin"]})
    except jwt_security.AgentIsNotAuthorized as e:
        assert e.message.startswith("Authorization token is not")
        return

    assert False, "Exc is not thrown"


@pytest.mark.asyncio
async def test_should_raise_not_authorized_error_when_token_is_expired():
    mock_handler.request.headers = {"Authorization": "Bearer: " + get_token(exp_minus=1000)}
    try:
        ret = await  manager._before_request_hook(mock_handler, {"secure": ["admin"]})
    except InvalidTokenError:
        assert True
        return
    assert False

@pytest.mark.asyncio
async def test_should_raise_invalid_token_error_when_token_issuer_not_match():
    mock_handler.request.headers = {"Authorization": "Bearer: " + get_token()}

    try:
        await  manager._before_request_hook(mock_handler, get_method_def())
    except InvalidTokenError:
        assert True
        return
    assert False, "Exception is not raised"


@pytest.mark.asyncio
async def test_should_not_raise_any_exception_when_issuer_is_match():
    mock_handler.request.headers = {"Authorization": "Bearer: " + get_token(issuer="Quark-CMS")}

    await  manager._before_request_hook(mock_handler, get_method_def())
    assert True, "No exception is not raised"


def test_partify_should():
    assert len(jwt_security.partify("application:create")) == 2
    assert len(jwt_security.partify("application,user_group:create")[0]) == 2
    assert len(jwt_security.partify("application:create:1,2,3")) == 3

def test_implies():
    assert jwt_security.implies("*", "application:read")
    assert not jwt_security.implies("application:read", "*")
    assert jwt_security.implies("application:*", "application:read")
    assert jwt_security.implies("application:read,write", "application:read")
    assert jwt_security.implies("one,two", "two")
    assert not jwt_security.implies("two", "one,two")
    assert jwt_security.implies("one,two:three,four,five:six", "one:three:six")
    assert not jwt_security.implies("one:three:six", "one,two:three,four,five:six")

def get_method_def():
    return {
        "secure": ["admin"]
    }


def get_token(exp_minus=0, issuer=None):
    jwt_payload = dict(
        exp=timegm(datetime.datetime.utcnow().utctimetuple()) - exp_minus,
        roles=["admin"],
        prn="hyurtseven"
    )

    if issuer:
        jwt_payload['iss'] = issuer

    return jwt.encode(jwt_payload, key=secret).decode("utf-8")








class CustomIDGenerator(object):

    def __init__(self, custom_id_collection):
        self._custom_id_collection = custom_id_collection

    async def generate(self, collection_name):
        custom_id = (await self._custom_id_collection.find_and_modify(
            query={'_id': collection_name},
            update={'$inc': {'custom_id': 1}},
            upsert=True,
            new=True
        )).get('custom_id', 0)

        return custom_id


import json

import logging
import pprint
import random
import string

from tornado.testing import AsyncHTTPTestCase

logger = logging.getLogger("testing.case")

class TestCaseError(Exception):
    pass

class APITestCase(AsyncHTTPTestCase):

    def get_settings(self):
        raise NotImplemented

    def generate_random_string(self, N):
        return ''.join(random.choice(
            string.ascii_uppercase + string.digits) for _ in range(N))

    @property
    def qap_api_url(self):
        return self.get_settings()["qap_api_url"]

    def fetch_ext(self, url, **kwargs):
        self.http_client.fetch(
            url,
            self.stop,
            **kwargs
        )

        return self.wait()

    def load_body(self, response):
        return json.loads(response.body.decode("utf-8"))

    def prettify(self, data):
        return pprint.pformat(data)

    def ensure_success(self, response):
        if response.code < 200 or response.code > 299:
            logger.error("API request failed.")
            logger.error(self.prettify(self.load_body(response)))
            raise TestCaseError

    def create_token(self, username, password, ensure_success=True):
        response = self.fetch_ext(
            self.qap_api_url + '/api/tokens',
            method='POST',
            body=json.dumps({
                "username": username,
                "password": password
            })
        )

        if ensure_success:
            self.ensure_success(response)

        return self.load_body(response)["token"]

    def create_membership(self, data, ensure_success=True):
        default_membership = {
                "_id": "cnnturk",
                "name": "CNNTurk",
                "username": "yusuf.sahin",
                "password": "brm_hy81"
        }

        default_membership.update(data)

        response = self.fetch_ext(
            self.qap_api_url + '/membership',
            method='POST',
            body=json.dumps(default_membership)
        )

        if ensure_success:
            self.ensure_success(response)

        self.membership = self.load_body(response)
        self.membership_creator_token = self.get_token_of(
            default_membership["username"], default_membership["password"])


    def create_user(self, data, ensure_success=True):
        default_user = {
            "username": "erdem.guner",
            "passowrd": "brm_hy81",
            "permissions": [],
            "domains": []
        }

        default_user.update(data)

        response = self.fetch_ext(
            '/api/users',
            method='POST',
            headers=self.get_auth_header(self.membership_creator_token),
            body=json.dumps(default_user)
        )

        if ensure_success:
            self.ensure_success(response)

        return response

    def create_domain(self, data, ensure_success=True):
        default_domain = {
            "name": self.generate_random_string(10)
        }

        default_domain.update(data)

        response = self.fetch_ext(
            self.qap_api_url + '/api/domains',
            method='POST',
            headers=self.get_auth_header(self.membership_creator_token),
            body=json.dumps(default_domain)
        )

        if ensure_success:
            self.ensure_success(response)

        return response

    def create_permission_group(self, data, ensure_success=True):
        default_permission_group = {
            "name": self.generate_random_string(10),
            "permissions": []
        }

        default_permission_group.update(data)

        response = self.fetch_ext(
            self.qap_api_url + '/api/permission-groups',
            method='POST',
            body=json.dumps(data),
            headers=self.get_auth_header(self.membership_creator_token)
        )

        if ensure_success:
            self.ensure_success(response)

        return response


    def test_create(self):
        self.get_membership_owner_token()





if __name__ == '__main__':
    test_case = APITestCase()
    test_case.run()





import json

import jwt
from bson import ObjectId

from quark_utilities import temporal_helpers
from quark_utilities.json_helpers import bson_to_json


class ObjectMother(object):

    WAYNE_ENTERPRISE = {
            "_id": "wayne",
            "name": "Wayne Enterprise",
            "sys": {
                "created_at": temporal_helpers.utc_now(),
                "created_by": "bruce.wayne",
                "cid": 1
            }
        }

    WAYNE_ENTERPRISE_SUBSCRIPTONS = [
        {
            "_id": ObjectId(),
            "product": "CMS",
            "membership_id": "wayne",
            "status": "ACTIVE",
            "api_permission_prefix": "cms",
            "sys": {
                "created_by": "bruce.wayne",
                "created_at": temporal_helpers.utc_now()
            }
        },
        {
            "_id": ObjectId(),
            "product": "QAP",
            "membership_id": "wayne",
            "status": "ACTIVE",
            "api_permission_prefix": "qap",
            "sys": {
                "created_by": "bruce.wayne",
                "created_at": temporal_helpers.utc_now()
            }
        }
    ]

    BRUCE_WAYNE = {
        "_id": "bruce.wayne",
        "password": "$2b$12$S4P.FaEeFcg.a8fFTSHowe0iWo3ZI5NDISdDgb6xTzktsnAfcvY9K",
        "membership_id": "wayne",
        "permissions": [
            "cms.*",
            "qap.*"
        ],
        "sys": {
            "created_by": "bruce.wayne",
            "created_at": temporal_helpers.utc_now(),
            "cid": 1
        }
    }

    ALFRED = {
        "_id": "alfred",
        "password": "$2b$12$S4P.FaEeFcg.a8fFTSHowe0iWo3ZI5NDISdDgb6xTzktsnAfcvY9K",
        "membership_id": "wayne",
        "permissions": [
            "cms.*",
            "qap.*"
        ],
        "sys": {
            "created_by": "bruce.wayne",
            "created_at": temporal_helpers.utc_now(),
            "cid": 2
        }
    }

    CIVIL = {
        "_id": "civil",
        "password": "$2b$12$S4P.FaEeFcg.a8fFTSHowe0iWo3ZI5NDISdDgb6xTzktsnAfcvY9K",
        "membership_id": "wayne",
        "permissions": [
        ],
        "sys": {
            "created_by": "bruce.wayne",
            "created_at": temporal_helpers.utc_now(),
            "cid": 3
        }
    }

    GOTHAM_NEWS = {
        "_id": ObjectId(),
        "name": "Gotham News",
        "membership_id": "wayne",
        "sys": {
            "created_by": "bruce.wayne",
            "created_at": temporal_helpers.utc_now(),
            "cid": 1
        }
    }

    VILLIANS_PERMISSON_GROUP = {
        "_id": ObjectId(),
        "name": "Gotham City Villians",
        "membership_id": "wayne",
        "permissions": [
            "qap.*"
        ],
        "sys": {
            "created_by": "bruce.wayne",
            "created_at": temporal_helpers.utc_now(),
            "cid": 1
        }
    }

    GOTHAM_NEWS_MASTER_TEMPLATE = {
        "_id": ObjectId(),
        "name": "Gotham News Master Page Template",
        "membership_id": "wayne",
        "domain_id": str(GOTHAM_NEWS["_id"]),
        "sys": {
            "created_by": "bruce.wayne",
            "created_at": temporal_helpers.utc_now(),
            "cid": 1
        }
    }

    GOTHAM_NEWS_FORM_COMPONENT = {
        "_id": ObjectId(),
        "name": "Gotham News Article Component",
        "type": "Article",
        "field_id": "test",
        "membership_id": "wayne",
        "domain_id": str(GOTHAM_NEWS["_id"]),
        "sys": {
            "created_by": "bruce.wayne",
            "created_at": temporal_helpers.utc_now(),
            "cid": 1
        }
    }

    GOTHAM_NEWS_CONTENT_TYPE = {
        "_id": ObjectId(),
        "name": "Gotham News Article Component",
        "type": "Article",
        "base_type": "test",
        "membership_id": "wayne",
        "domain_id": str(GOTHAM_NEWS["_id"]),
        "sys": {
            "created_by": "bruce.wayne",
            "created_at": temporal_helpers.utc_now(),
            "cid": 1
        }
    }

    def __init__(self, mongo_client):
        self._mongo_client = mongo_client

    def create_membership(self):
        self._mongo_client.quark_test.memberships.save(self.WAYNE_ENTERPRISE)

        self._mongo_client.quark_test.users.insert(self.BRUCE_WAYNE)

        for sub in self.WAYNE_ENTERPRISE_SUBSCRIPTONS:
            self._mongo_client.quark_test.subscriptions.save(sub)

        return (self.WAYNE_ENTERPRISE, self.BRUCE_WAYNE, self.WAYNE_ENTERPRISE_SUBSCRIPTONS)

    def create_bruce_wayne_token(self, secret):
        payload = {
            "prn": self.BRUCE_WAYNE["_id"],
            "membership_id": self.BRUCE_WAYNE["membership_id"],
            "permissions": self.BRUCE_WAYNE["permissions"],
            "domains":[],
            'exp': 99999999999,
            'jti': str(ObjectId()),
            'iat': temporal_helpers.to_timestamp(temporal_helpers.utc_now())
        }

        payload = json.loads(json.dumps(payload, default=bson_to_json))

        return jwt.encode(payload, secret).decode("utf-8")

    def create_user_civil(self):
        self._mongo_client.quark_test.users.save(self.CIVIL)
        return self.CIVIL

    def create_civil_token(self, secret):
        payload = {
            "prn": self.CIVIL["_id"],
            "membership_id": self.CIVIL["membership_id"],
            "permissions": self.CIVIL["permissions"],
            "domains": [],
            'exp': 99999999999,
            'jti': str(ObjectId()),
            'iat': temporal_helpers.to_timestamp(temporal_helpers.utc_now())
        }

        payload = json.loads(json.dumps(payload, default=bson_to_json))

        return jwt.encode(payload, secret).decode("utf-8")

    def create_user_alfred(self):
        self._mongo_client.quark_test.users.save(self.ALFRED)
        return self.ALFRED

    def create_domain_gotham_news(self):
        self._mongo_client.quark_test.domains.save(self.GOTHAM_NEWS)
        return self.GOTHAM_NEWS

    def create_template_for_gotham_news(self):
        self._mongo_client.quark_test.templates.save(self.GOTHAM_NEWS_MASTER_TEMPLATE)
        return self.GOTHAM_NEWS_MASTER_TEMPLATE

    def create_form_component_for_gotham_news(self):
        self._mongo_client.quark_test.form_components.save(self.GOTHAM_NEWS_FORM_COMPONENT)
        return self.GOTHAM_NEWS_FORM_COMPONENT

    def create_content_type_for_gotham_news(self):
        self._mongo_client.quark_test.content_types.save(self.GOTHAM_NEWS_CONTENT_TYPE)
        return self.GOTHAM_NEWS_CONTENT_TYPE

    def create_permission_group(self):
        self._mongo_client.quark_test.permission_groups.save(
            self.VILLIANS_PERMISSON_GROUP)
        return self.VILLIANS_PERMISSON_GROUP

    def drop(self):
        self._mongo_client.drop_database("quark_test")



import functools
import json
from urllib.parse import urljoin

import bcrypt
import pymongo

from quark_utilities import json_helpers
from tornado.ioloop import IOLoop
from tornado.platform.asyncio import AsyncIOMainLoop
from tornado.testing import AsyncHTTPTestCase

from quark_utilities.commons.testing.object_mother import ObjectMother

class TestConfigurationError(Exception):
    pass


class QuarkBase(AsyncHTTPTestCase):

    DEFAULT_SETTINGS = {
        "secret": bcrypt.gensalt(),
        "token_ttl": 30000,
        "token_issuer": "TestManager",
        "mongo_connection_string": "localhost",
        "mongo_default_db": "quark_test",
        "save_events": True
    }

    def setUp(self):
        super(QuarkBase, self).setUp()
        self.object_mother = ObjectMother(pymongo.MongoClient("localhost"))
        self.object_mother.drop()

        for on_start_func in self.flasky_app.on_start_funcs:
            IOLoop.current().run_sync(functools.partial(on_start_func, self.flasky_app))

        if hasattr(self, "before_t"):
            self.before_t()

    def get_new_ioloop(self):
        self._io_loop = AsyncIOMainLoop()
        return self._io_loop

    def get_app(self):
        self.flasky_app = self.get_flasky_app()
        self.flasky_app.build_app()
        return self.flasky_app.app

    def get_settings(self):
        return self.DEFAULT_SETTINGS

    def fetch_ext(self, url, **kwargs):
        self.http_client.fetch(
            url,
            self.stop,
            **kwargs
        )

        return self.wait()

    def assertErrorCodeEquals(self, response, code, msg=None):
        body = self.load_body(response)
        self.assertEquals(body["error"]["code"], code, msg=msg)

    def load_body(self, response):
        return json.loads(response.body.decode("utf-8"))

    def craft_fields(self):
        data = {}
        for key, val in self.required_fields:
            data[key] = val
        return data




class ResourceCreationCase(QuarkBase):

    def do_post(self, body):
        headers = {}

        if hasattr(self, "token"):
            headers.update({
                "Authorization": "Bearer " + self.token
            })

        return self.fetch(
            self.resource_url,
            method='POST',
            headers=headers,
            body=json.dumps(body)
        )

    def test_201(self):
        response = self.do_post(body=self.craft_fields())

        self.assertEquals(response.code, 201)
        ret = json.loads(response.body.decode("utf-8"))
        self.assertIsNotNone(ret["_id"])
        self.assertIsNotNone(ret["sys"])
        self.assertIsNotNone(ret["sys"]["created_at"])
        self.assertIsNotNone(ret["sys"]["created_by"])
        self.assertIsNotNone(ret["sys"]["cid"])


    def test_400(self):
        for key, _ in self.required_fields:
            body = self.craft_fields()

            body.pop(key, None)

            response = self.do_post(body=body)

            self.assertEquals(response.code, 400)
            self.assertErrorCodeEquals(response, "errors.parameterRequired")

    def test_403(self):
        if not hasattr(self, "civil_token"):
            return

        response = self.do_post(body=self.craft_fields())
        self.assertEquals(response.code, 403)
        self.assertErrorCodeEquals(response, "errors.agentIsNotAuthorized")

    def test_409(self):
        body = self.craft_fields()

        response = self.do_post(body=body)
        self.assertEquals(response.code, 201)

        response = self.do_post(body=body)
        self.assertEquals(response.code, 409)


class ResourceUpdateCase(QuarkBase):

    def do_put(self, _id, body, token):
        headers = {
            "Authorization": "Bearer " + token
        }

        return self.fetch(
            urljoin(self.resource_url,str(_id)),
            method='PUT',
            headers=headers,
            body=json.dumps(body, default=json_helpers.bson_to_json)
        )

    def test_200(self):
        resource = self.resource
        resource["test_key"] = "test_val"

        response = self.do_put(resource["_id"], resource, self.token)

        self.assertEquals(response.code, 200)
        ret = json.loads(response.body.decode("utf-8"))
        self.assertIsNotNone(ret["_id"])
        self.assertEquals(ret["test_key"], "test_val")
        self.assertIsNotNone(ret["sys"])
        self.assertIsNotNone(ret["sys"]["created_at"])
        self.assertIsNotNone(ret["sys"]["created_by"])
        self.assertIsNotNone(ret["sys"]["cid"])
        self.assertIsNotNone(ret["sys"]["modified_at"])
        self.assertIsNotNone(ret["sys"]["modified_by"])

    def test_403(self):
        if hasattr(self, "tests") and "403" not in self.tests:
            self.skipTest("Not running")

        if not hasattr(self, "token"):
            raise TestConfigurationError("This test requires token...")

        self.object_mother.create_user_civil()
        token = self.object_mother.create_civil_token(self.DEFAULT_SETTINGS["secret"])

        response = self.do_put(self.resource["_id"], self.resource, token)

        self.assertEquals(response.code, 403)

        code = getattr(self, "error_code_403", "errors.notAuthorized")
        self.assertErrorCodeEquals(response, code)


class ResourceDeleteCase(QuarkBase):

    def do_delete(self, _id, token):
        headers = {
            "Authorization": "Bearer " + token
        }

        return self.fetch(
            urljoin(self.resource_url, str(_id)),
            method='DELETE',
            headers=headers
        )

    def test_204(self):
        if hasattr(self, "tests") and "204" not in self.tests:
            self.skipTest("Not running")

        if not hasattr(self, "token"):
            raise TestConfigurationError("This test requires token...")

        response = self.do_delete(self.resource_id, token=self.token)

        self.assertEquals(response.code, 204)

    def test_404(self):
        if hasattr(self, "tests") and "404" not in self.tests:
            self.skipTest("Not running")

        if not hasattr(self, "token"):
            raise TestConfigurationError("This test requires token...")


        response = self.do_delete("123456789", token=self.token)
        self.assertEquals(response.code, 404)

    def test_403(self):
        if hasattr(self, "tests") and "403" not in self.tests:
            self.skipTest("Not running")

        self.object_mother.create_user_civil()
        token = self.object_mother.create_civil_token(self.DEFAULT_SETTINGS["secret"])

        response = self.do_delete(self.resource_id, token=token)
        self.assertEquals(response.code, 403)



class ResourceFindCase(QuarkBase):

    def test_200(self):
        if hasattr(self, "tests") and "200" not in self.tests:
            self.skipTest("Not running")

        if not hasattr(self, "token"):
            raise TestConfigurationError("This test requires token...")

        headers = {
            "Authorization": "Bearer " + self.token
        }

        response = self.fetch(
            urljoin(self.resource_url, str(self.resource_id)),
            headers=headers
        )

        print(response.body)
        self.assertEquals(response.code, 200)

        ret = json.loads(response.body.decode("utf-8"))
        self.assertEquals(ret["_id"], self.resource_id)

    def test_403(self):
        if hasattr(self, "tests") and "403" not in self.tests:
            self.skipTest("Not running")

        self.object_mother.create_user_civil()
        token = self.object_mother.create_civil_token(self.DEFAULT_SETTINGS["secret"])

        headers = {
            "Authorization": "Bearer " + token
        }

        response = self.fetch(
            urljoin(self.resource_url, str("1234567890")),
            headers=headers
        )

        code = "errors.notAuthorized"
        if hasattr(self, "error_code_403"):
            code = self.error_code_403

        self.assertEquals(response.code, 403)
        self.assertErrorCodeEquals(response, code)


    def test_404(self):
        if hasattr(self, "tests") and "404" not in self.tests:
            self.skipTest("Not running")

        if not hasattr(self, "token"):
            raise TestConfigurationError("This test requires token...")

        headers = {
            "Authorization": "Bearer " + self.token
        }

        response = self.fetch(
            urljoin(self.resource_url, str("1234567890")),
            headers=headers
        )

        self.assertEquals(response.code, 404)
        self.assertErrorCodeEquals(response, "errors.resourceNotFound")


from bson.json_util import loads
import logging


def get_skip(handler):
    skip = handler.get_argument('skip', 0)
    return int(skip)


def get_limit(handler, check_max_limit=True):
    max_limit = 1000
    limit = 0
    limit = handler.get_argument('limit', None)
    if limit:
        if check_max_limit and int(limit) > max_limit:
            limit = max_limit
        return int(limit)


def get_select(handler):
    select = None
    select_param = handler.get_argument('select', None)
    if select_param:
        select = {}
        fields = select_param.split(',')
        for field in fields:
            field = field.strip()
            if field:
                select[field] = 1

    return select


def get_where(handler):
    where = None
    where = handler.request.body.decode("utf-8")
    if where:
        try:
            json = loads(where)
            return json
        except Exception as ex:
            logging.error(ex)
            return None


def get_sort(handler):
    sort = None
    order_by = handler.get_argument('sort', None)
    if order_by:
        sort = []
        clauses = order_by.split(',')
        for clause in clauses:
            clause = clause.strip()
            if clause:
                if ' ' not in clause:
                    clause = clause + ' asc'
                sort.append(
                    (clause.split(' ')[0], -1 if clause.split(' ')[1] == 'desc' else 1))

    return sort



import collections


def update(d, u):
    for k, v in u.iteritems():
        if isinstance(v, collections.Mapping):
            r = update(d.get(k, {}), v)
            d[k] = r
        else:
            if not d:
                d = {}
            d[k] = u[k]
    return d


class DictDiffer(object):
    """
    Calculate the difference between two dictionaries as:
    (1) items added
    (2) items removed
    (3) keys same in both but changed values
    (4) keys same in both and unchanged values
    """

    def __init__(self, current_dict, past_dict):
        self.current_dict, self.past_dict = current_dict, past_dict
        self.set_current, self.set_past = set(
            current_dict.keys()), set(past_dict.keys())
        self.intersect = self.set_current.intersection(self.set_past)

    def added(self):
        return self.set_current - self.intersect

    def removed(self):
        return self.set_past - self.intersect

    def changed(self):
        return set(o for o in self.intersect if self.past_dict[o] != self.current_dict[o])

    def unchanged(self):
        return set(o for o in self.intersect if self.past_dict[o] == self.current_dict[o])



import datetime
import decimal


from bson import ObjectId
from bson.json_util import default


#: TODO delete these functions
def object_hook(json_dict):
    for (key, value) in json_dict.items():
        try:
            json_dict[key] = datetime.datetime.strptime(
                value, "%Y-%m-%dT%H:%M:%S.%fZ")
        except:
            pass
    return json_dict


def bson_to_json(o):
    if isinstance(o, ObjectId):
        return str(o)
    if isinstance(o, datetime.datetime):
        r = o.isoformat()
        return r + 'Z'
    elif isinstance(o, datetime.date):
        return o.isoformat()
    elif isinstance(o, datetime.time):
        r = o.isoformat()
        if o.microsecond:
            r = r[:12]
        return r
    elif isinstance(o, decimal.Decimal):
        return str(o)
    return default(o)



import pymongo
from bson import ObjectId
from flasky.errors import FlaskyTornError
from pymongo.errors import DuplicateKeyError

__all__ = ["maybe_object_id", "Repository", "create_id"]


class RecordNotExistsError(FlaskyTornError):
    def __init__(self, record_type, record_id):
        super().__init__(status_code=404, err_code="errors.resourceNotFound")
        self.record_type = record_type
        self.record_id = record_id

class IDMismatchError(FlaskyTornError):
    def __init__(self, given_id, data_id):
        super().__init__(message="errors.idMismatchError", err_code="errors.idMismatchError", status_code=400)
        self.given_id = given_id
        self.data_id = data_id

class FlaskyDuplicateKeyError(FlaskyTornError):
    def __init__(self, collection_name, key, pymongo_exc=None):
        super().__init__(status_code=409,
                         message="There is already existing record at <{}> collection"
                                 "with given <{}>key ".format(collection_name, key),
                         err_code="errors.resourceAlreadyExists",
                         reason=pymongo_exc)
        self.collection_name = collection_name
        self.key = key


def maybe_object_id(maybe_id):
    if isinstance(maybe_id, ObjectId):
        return maybe_id
    elif not ObjectId.is_valid(maybe_id):
        return maybe_id
    else:
        return ObjectId(maybe_id)

def create_id():
    return ObjectId()

class Repository(object):

    def __init__(self, collection):
        self.collection = collection

    @property
    def collection_name(self):
        return self.collection.name

    async def query(self, select=None, where=None,
                    sort=None, skip=None, limit=None, return_count=False):
        cur = self.collection.find(where, select)
        count  = await cur.count()

        if skip:
            cur.skip(skip)

        if sort:
            cur.sort(sort)

        if limit:
            cur.limit(limit)

        records = await cur.to_list(length=None)

        if return_count:
            return records, count

        return [record
                for record in records
                if not record.get("is_deleted", False)]

    async def save(self, data):
        if not data.get("_id", None):
            # ~append id if it not exists
            data["_id"] = ObjectId()

        try:
            await self.collection.insert_one(data)
            return data
        except DuplicateKeyError as e:
            raise FlaskyDuplicateKeyError(
                collection_name=self.collection_name,
                key=data.get("_id"),
                pymongo_exc=e
            )

    async def update(self, _id, data):
        #: Data object should not have id field in it
        #: if it has it should be the same id with given
        #: id
        if "_id" in data:
            if str(_id) != str(data["_id"]):
                raise IDMismatchError(_id, data["_id"])

            data["_id"] = maybe_object_id(data["_id"])

        try:
            await self.collection.replace_one({
                "_id": maybe_object_id(_id)
            }, data)
            return data
        except DuplicateKeyError as e:
            raise FlaskyDuplicateKeyError(
                collection_name=self.collection_name,
                key=data.get("_id"),
                pymongo_exc=e
            )

    async def delete(self, _id):
        record = await self.find(_id=_id)
        if not record:
            raise RecordNotExistsError(self.collection.name, _id)
        await self.collection.remove(record["_id"], record)

    async def delete_one_by(self, where=None):
        record = await self.find_one_by(where=where)
        if not record:
            raise RecordNotExistsError(self.collection.name, where.get("_id", None))

        if "_id" in where:
            where["_id"] = maybe_object_id(where=where)

        await self.collection.deleteOne(where=where)

    async def delete_many_by(self, where=None):
        record = await self.find_one_by(where=where)
        if not record:
            raise RecordNotExistsError(self.collection.name, where.get("_id", None))

        await self.collection.deleteMany(where=where)

    async def find(self, _id=None, raise_exec=True):
        document = await self.collection.find_one({"_id": maybe_object_id(_id)})
        if not document and raise_exec:
            raise RecordNotExistsError(self.collection_name, _id)
        return document

    async def find_one_by(self, where=None, select=None, raise_exec=True):
        if "_id" in where:
            where["_id"] = maybe_object_id(where["_id"])
        document = await self.collection.find_one(where, select)
        if not document and raise_exec:
            raise RecordNotExistsError(self.collection_name, where)
        return document




import json
import logging

from quark_utilities import json_helpers

from quark_utilities import mongol

from flasky.errors import FlaskyTornError
from quark_utilities.json_schema import JSONSchemaValidationError
from quark_utilities.jwt_security import AgentIsNotAuthorized
from quark_utilities.mongol import FlaskyDuplicateKeyError, IDMismatchError

logger = logging.getLogger("error.handlers")


class InfrastructureError(FlaskyTornError):

    def __init__(self, message=None):
        super().__init__(status_code=500, message=message)


def set_commons(handler, err):
    handler.set_status(err.status_code)
    handler.set_header("Content-Type", "application/json")


def init_error_handlers(app, settings):

    @app.error_handler(AgentIsNotAuthorized)
    async def handle_auth_error(handler, err, method_definition):
        handler.write({
            "error": {
                "code": err.err_code,
                "errors": [err.message],
                "required_permission": err.required_permission,
            }
        })
        set_commons(handler, err)

    @app.error_handler(FlaskyDuplicateKeyError)
    async def handle_duplicate_key_err(handler, err, method_definition):
        handler.write(json.dumps({
            "error": {
                "code": err.err_code,
                "errors": [err.message],
                "collection": err.collection_name,
                "key": err.key
            }
        }, default=json_helpers.bson_to_json))
        set_commons(handler, err)

    @app.error_handler(mongol.RecordNotExistsError)
    async def handle_record_exist_error(handler, err, method_definition):
        handler.write({
            "error": {
                "code": err.err_code,
                "errors": [err.message],
                "key": err.record_id,
                "collection": err.record_type
            }
        })
        set_commons(handler, err)

    @app.error_handler(JSONSchemaValidationError)
    async def handle_json_schema_error(handler, err, method_definition):
        handler.write({
            "error": {
                "code": "errors.parameterRequired",
                "errors": [err.message],
                "schema": method_definition.get("json_schema", {})
            }
        })
        set_commons(handler, err)

    @app.error_handler(IDMismatchError)
    async def handle_id_mismatch_err(handler, err, method_definition):
        handler.write({
            "error": {
                "code": "errors.badRequest",
                "message": "Given ID<{}> and Payload ID<{}> is different.".format(
                    err.given_id, err.data_id),
            }
        })

    @app.error_handler()
    async def handle_errors(handler, err, method_definition):
        logging.exception("Internal error occured")
        if isinstance(FlaskyTornError, err):
            handler.write({
                'error': {
                    'message': err.message,
                    'errors': []
                }
            })
            handler.set_status(err.status_code)